"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var path_1 = require("path");
var node_fetch_1 = require("node-fetch");
var consola = require('consola');
var child_process_1 = require("child_process");
var util_1 = require("./util");
var logger = consola.withScope('Runner');
var TestRunner = /** @class */ (function () {
    function TestRunner(engine, config, preference) {
        this.engine = engine;
        this.config = config;
        this.preference = preference;
    }
    TestRunner.prototype.start = function (watch, disableCoverageCollection, testFile, testName) {
        if (watch === void 0) { watch = false; }
        if (disableCoverageCollection === void 0) { disableCoverageCollection = true; }
        if (testFile === void 0) { testFile = ''; }
        if (testName === void 0) { testName = ''; }
        var patchJsFile = path_1.join(__dirname, './patch.js');
        var repoterPath = path_1.join(__dirname, './reporter.js');
        var loggerPath = path_1.join(__dirname, './logger.js');
        // const teardown = join(__dirname, './teardown.js');
        var NodeExecutable = this.preference.getNodePath()
            ? this.preference.getNodePath()
            : 'node';
        var setupFilesArg = (this.preference.getMajesticLogEnabled() ? [loggerPath] : []).concat((this.config.jest.setupFiles || []));
        var jestScript = path_1.join(this.engine.root, this.config.jestScript);
        logger.success("Starting jest with:\n                      \uD83D\uDCC8 Coverage collection: " + !disableCoverageCollection);
        this.jestProcess = child_process_1.spawn(NodeExecutable + " -r " + patchJsFile + " " + jestScript, (watch ? ['--watchAll'] : []).concat((testName
            ? ['--testNamePattern', util_1.getTestPatternForTestName(testName)]
            : []), (testFile ? [util_1.getTestPatternForPath(testFile)] : []), ['--reporters', 'default', repoterPath], ['--setupFiles'].concat(setupFilesArg), (disableCoverageCollection ? [] : ['--collectCoverage=false']), (this.config.args ? this.config.args : [])
        // ...['--globalTeardown', teardown]
        ), {
            cwd: this.engine.root,
            shell: true,
            stdio: 'pipe',
            env: this.getEnvironment()
        });
        this.jestProcess.stdout.on('data', function (data) {
            consola.info(data.toString().trim());
        });
        this.jestProcess.stderr.on('data', function (data) {
            consola.info(data.toString().trim());
        });
        this.jestProcess.on('exit', function (code) {
            consola.warn('Jest process did exit');
        });
    };
    TestRunner.prototype.updateSnapshot = function (testFile, testName) {
        var _this = this;
        return new Promise(function (resolve) {
            var patchJsFile = path_1.join(__dirname, './patch.js');
            var loggerPath = path_1.join(__dirname, './mockLogger.js');
            var setupFilesArg = [
                loggerPath
            ].concat((_this.config.jest.setupFiles || []));
            var updateProcess = child_process_1.spawn("node -r " + patchJsFile + " " + path_1.join(_this.engine.root, _this.config.jestScript) + " ", ['--setupFiles'].concat(setupFilesArg).concat([
                '--updateSnapshot'
            ], (testName
                ? ['--testNamePattern', util_1.getTestPatternForTestName(testName)]
                : []), (testFile ? [util_1.getTestPatternForPath(testFile)] : [])), {
                cwd: _this.engine.root,
                shell: true,
                stdio: 'pipe',
                env: _this.getEnvironment()
            });
            updateProcess.on('close', function () {
                resolve(JSON.stringify({}));
            });
        });
    };
    TestRunner.prototype.startInspect = function (testFile, testName) {
        var _this = this;
        return new Promise(function (resolve) {
            // kill the existing inspect process
            _this.kill(_this.inspectProcess);
            var inspectProcess = child_process_1.spawn("node --inspect-brk " + path_1.join(_this.engine.root, _this.config.jestScript) + " ", (testName
                ? ['--testNamePattern', testName.replace(/\s/g, '.')]
                : []).concat((testFile ? [util_1.getTestPatternForPath(testFile)] : [])), {
                cwd: _this.engine.root,
                shell: true,
                stdio: 'pipe',
                env: _this.getEnvironment()
            });
            inspectProcess.on('close', function () {
                if (_this.onDebuggerExit) {
                    _this.onDebuggerExit();
                }
            });
            _this.getDebuggerUrl().then(function (url) {
                resolve(JSON.stringify({
                    url: url
                }));
            });
        });
    };
    TestRunner.prototype.registerOnDebuggerExit = function (callback) {
        this.onDebuggerExit = callback;
    };
    TestRunner.prototype.kill = function (processToKill) {
        if (processToKill === void 0) { processToKill = this.jestProcess; }
        if (!processToKill) {
            return;
        }
        if (process.platform === 'win32') {
            // Windows doesn't exit the process when it should.
            child_process_1.spawn('taskkill', ['/pid', '' + processToKill.pid, '/T', '/F']);
        }
        else {
            processToKill.kill();
        }
    };
    TestRunner.prototype.runTestByFileInteractive = function (testFileName) {
        var _this = this;
        util_1.executeInSequence([
            {
                fn: function () { return _this.jestProcess.stdin.write('p'); },
                delay: 0
            },
            {
                fn: function () {
                    return _this.jestProcess.stdin.write(util_1.getTestPatternForPath(testFileName));
                },
                delay: 100
            },
            {
                fn: function () {
                    return _this.jestProcess.stdin.write(new Buffer('0d', 'hex').toString());
                },
                delay: 200
            }
        ]);
    };
    TestRunner.prototype.runTestByTestNameInteractive = function (testFileName, testName) {
        var _this = this;
        util_1.executeInSequence([
            {
                fn: function () { return _this.jestProcess.stdin.write('t'); },
                delay: 200
            },
            {
                fn: function () { return _this.jestProcess.stdin.write(testName); },
                delay: 500
            },
            {
                fn: function () {
                    return _this.jestProcess.stdin.write(new Buffer('0d', 'hex').toString());
                },
                delay: 200
            }
        ]);
    };
    TestRunner.prototype.getJestExecutablePath = function () {
        return this.config.jestScript;
    };
    TestRunner.prototype.getEnvironment = function () {
        var env = process.env || {};
        if (this.config.env) {
            for (var _i = 0, _a = Object.entries(this.config.env); _i < _a.length; _i++) {
                var _b = _a[_i], key = _b[0], value = _b[1];
                env[key] = value;
            }
        }
        return env;
    };
    TestRunner.prototype.getDebuggerUrl = function () {
        return node_fetch_1.default('http://localhost:9229/json')
            .then(function (response) {
            return response.json();
        })
            .then(function (debugInfo) {
            return "chrome-devtools://devtools/bundled/inspector.html?experiments=true&v8only=true&ws=localhost:9229/" + debugInfo[0].id;
        });
    };
    return TestRunner;
}());
exports.default = TestRunner;
//# sourceMappingURL=Runner.js.map