"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var invariant = require("invariant");
var isPromise = require('p-is-promise');
var InvocationHandler = /** @class */ (function () {
    function InvocationHandler(socket) {
        this.remotables = new Map();
        this.socket = socket;
    }
    InvocationHandler.prototype.setSocket = function (emitter, listener) {
        var _this = this;
        this.emitter = emitter;
        this.listener = listener;
        this.remotables.forEach(function (remotable, name) {
            _this.emitMethods({
                extensionName: name,
                methods: Object.keys(remotable.localMethods)
            });
        });
        this.listener.on('methods', function (methods) {
            _this.buildRemoteMethodsInvoker(methods);
        });
        this.listener.on('result', function (result) {
            _this.callResolverWithResult(result);
        });
        this.listener.on('invoke', function (payload) {
            _this.invokeLocalMethod(payload);
        });
    };
    InvocationHandler.prototype.registerExtension = function (extensionName, localMethods, remoteReady) {
        this.remotables.set(extensionName, {
            localMethods: localMethods,
            remoteReady: remoteReady
        });
        this.emitMethods({
            extensionName: extensionName,
            methods: Object.keys(localMethods)
        });
    };
    InvocationHandler.prototype.buildRemoteMethodsInvoker = function (_a) {
        var _this = this;
        var extensionName = _a.extensionName, methods = _a.methods;
        var remote = {};
        methods.forEach(function (method) {
            remote[method] = function () {
                var args = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    args[_i] = arguments[_i];
                }
                return new Promise(function (resolve, reject) {
                    _this.setResolverForAMethod(extensionName, method, resolve);
                    _this.emitInvoke(extensionName, method, args);
                });
            };
        });
        var remotable = this.remotables.get(extensionName);
        if (remotable && remotable.remoteReady) {
            remotable.remoteMethods = remote;
            remotable.remoteReady(remote);
        }
    };
    InvocationHandler.prototype.emitMethods = function (methods) {
        this.emitter.emit('methods', methods);
    };
    InvocationHandler.prototype.emitInvoke = function (extensionName, method, args) {
        invariant(this.socket, 'Socket is not available to emit 😭');
        this.emitter.emit('invoke', {
            extensionName: extensionName,
            methodName: method,
            args: args
        });
    };
    InvocationHandler.prototype.emitResult = function (extensionName, methodName, result) {
        this.emitter.emit('result', {
            extensionName: extensionName,
            methodName: methodName,
            result: result
        });
    };
    InvocationHandler.prototype.setResolverForAMethod = function (extensionName, methodName, resolver) {
        var remotable = this.remotables.get(extensionName);
        if (remotable) {
            remotable.resolvers = remotable.resolvers || {};
            remotable.resolvers[methodName] = resolver;
        }
    };
    InvocationHandler.prototype.callResolverWithResult = function (_a) {
        var extensionName = _a.extensionName, methodName = _a.methodName, result = _a.result;
        var extension = this.remotables.get(extensionName);
        if (extension) {
            var resolver = extension.resolvers[methodName];
            resolver(JSON.parse(result));
        }
    };
    InvocationHandler.prototype.invokeLocalMethod = function (_a) {
        var _this = this;
        var extensionName = _a.extensionName, methodName = _a.methodName, args = _a.args;
        var extension = this.remotables.get(extensionName);
        if (!extension) {
            return;
        }
        var method = extension.localMethods[methodName];
        var handlerResult = method.apply(void 0, args);
        if (isPromise(handlerResult)) {
            handlerResult.then(function (result) {
                _this.emitResult(extensionName, methodName, result);
            });
        }
        else {
            this.emitResult(extensionName, methodName, handlerResult);
        }
    };
    return InvocationHandler;
}());
exports.default = InvocationHandler;
//# sourceMappingURL=InvocationHandler.js.map